-- ESFA: InitUpdateLookup
-- Demonstrate initialization, update, lookup, and printing arrays
-- This program is discussed in doc/src/esfa-users-guide.txt

module Main where

import Control.Monad.IO.Class
import Data.Array.ESFA.SimArch

main :: IO ()
main = simulateESFM 10 () script

script :: StESFA () Int ()
script =
  do liftIO (putStrLn "ESFA Example InitUpdateLookup")
     esfmInitialize
     esfmSetOption EsfmTraceOperation

-- Create a couple of arrays
     Just a1 <- esfmUpdate "a1" esfmEmpty (100, 101)
     Just a2 <- esfmUpdate "a2" a1 (200, 201)
-- Expected contents of the arrays:
--     a1 = 100~>101
--     a2 = 200~>201, 100~>101 

-- Fetch and print some array elements
     Just x1 <- esfmLookup a1 100   -- expect 101
     Just x2 <- esfmLookup a2 100   -- expect 101
     Just x3 <- esfmLookup a2 200   -- expect 201
     liftIO (putStrLn (show [x1,x2,x3]))

-- Fetch and print some non-existent array elements
     y1 <- esfmLookup a1 200     -- expect Nothing
     y2 <- esfmLookup a2 300     -- expect Nothing
     liftIO (putStrLn (show [y1,y2]))

-- Print the expected and then the actual arrays
     esfmPrintSpecArrays
     esfmPrintMachineArrays
