-- ESFA example: RepSharing
-- Show how ESF arrays are represented and demonstrate sharing of elements
-- This program is discussed in doc/src/esfa-users-guide.txt

module Main where

import Control.Monad.IO.Class
import Data.Array.ESFA.SimArch

main :: IO ()
main = simulateESFM 10 () script

showState =
  do esfmPrintState
     esfmPrintMachineArrays
     esfmPrintSpecArrays
     stats <- esfmAnalyzeMachineState
     liftIO (putStrLn (show stats))
     liftIO (putStrLn (take 75 (repeat '-')))

script :: StESFA () Int ()
script =
  do liftIO (putStrLn "ESFA Example: RepSharing")
     esfmInitialize
     esfmSetOption EsfmTraceOperation

     Just a1 <- esfmUpdate "a1" esfmEmpty (100, 101)
-- a1 = 100-101
     showState

     Just a2 <- esfmUpdate "a2" a1 (200, 201)
-- a2 = 200-201 100-101 
     showState

     Just a3 <- esfmUpdate "a3" a2 (300, 301)
-- a3 = 300-301 200-201 100-101 
     showState

     Just a4 <- esfmUpdate "a4" a2 (400, 401)
-- a4 = 400-401 200-201 100-101 
     showState

     Just a5 <- esfmUpdate "a5" a2 (500, 501)
-- a5 = 500-501 200-201 100-101 
     showState

     Just a6 <- esfmUpdate "a6" a3 (600, 601)
-- a6 = 600-601 300-301 200-201 100-101 
     showState

     Just a7 <- esfmUpdate "a7" a5 (700, 701)
-- a7 = 700-701 500-501 200-201 100-101 
     showState

     Just a8 <- esfmUpdate "a8" a1 (800, 801)
-- a8 = 800-801 100-101
     showState

     Just a9 <- esfmUpdate "a9" esfmEmpty (900, 901)
-- a9 = 900-901
     showState
